<?php
/*
 * Horde Configuration File
 *
 * This file contains global configuration settings for Horde.  Values may be
 * safely edited by hand.  Use horde.php.dist as a reference.
 *
 * Default user preferences are defined in 'prefs.php'.
 *
 * Strings should be enclosed in 'quotes'.
 * Integers should be given literally (without quotes).
 * Boolean values may be 'true' or 'false'.
 *
 * $Horde: horde/config/horde.php.dist,v 1.47.2.18 2002/05/18 19:18:56 jon Exp $
 */

/**
 ** General Horde Settings
 **/

// The value to set error_reporting() to. Valid values are: E_ERROR,
// E_WARNING, E_PARSE, E_NOTICE, E_CORE_ERROR, E_CORE_WARNING,
// E_ALL. See http://www.php.net/manual/function.error-reporting.php
// for more information.
$conf['debug_level'] = E_ALL;

// If we need to perform a long operation, what should we set
// max_execution_time to (in seconds)? 0 means no limit; however, a
// value of 0 will cause a warning if you are running in safe
// mode. See http://www.php.net/manual/function.set-time-limit.php for
// more information.
$conf['max_exec_time'] = 0;

// What name should we use for the session that Horde apps share? If
// you want to share sessions with other applications on your
// webserver, you will need to make sure that they are using the same
// session name.
$conf['session_name'] = 'Horde';

// Determines how we generate full URLs (for location headers and
// such). Possible values are:
//   0 - Assume that we are not using SSL and never generate https URLS.
//   1 - Assume that we are using SSL and always generate https URLS.
//   2 - Attempt to auto-detect, and generate URLs appropriately.
$conf['use_ssl'] = 2;

// If this option is set to true, and you have the php zlib extension,
// pages over a certain size will be compressed and sent to the
// browser as gzip-encoded data in order to save bandwidth. There is a
// CPU-usage penalty to pay for this, but the decrease in page size
// can be dramatic (70k to under 10k for a full mailbox page), and is
// more than worth it over anything but an extremely fast link.
$conf['compress_pages'] = true;

// What umask should we run with? This will affect the permissions on
// any temporary files that are created. This value is an integer
// (specify it WITHOUT quotes).
$conf['umask'] = 077;

// If you want to use a temporary directory other than the system
// default or the one specified in php's upload_tmp_dir value, enter
// it here.
$conf['tmpdir'] = null;


/**
 ** Horde Authentication
 **/

// If you want to use IMP with Horde and don't want to let the users
// login twice (once for Horde and once for IMP) you can setup Horde
// to let IMP do the authentication stuff. Just uncomment the
// auth/login/logout lines in registry.php instead of configuring the
// following settings.

// What backend should we use for authenticating users to Horde? Valid
// options are currently 'imap', 'ldap', 'mcal', 'sql', and 'ftp'.
$conf['auth']['driver'] = 'imap';

// An array holding any parameters that the Auth object will need to
// function correctly.
$conf['auth']['params'] = array();

// For IMAP, this is the server name, port, protocol, etc.
// $conf['auth']['params']['dsn'] = '{imap.example.com:143/imap}INBOX';

// See $conf['prefs']['params'] further down for an example how to
// setup a SQL backend. But you must use the horde_users SQL table for
// the authentication driver. A SQL script can be found in
// horde/scripts/db/auth.sql.


/**
 ** Horde Logging
 **/

// Should Horde log errors and other useful information?
$conf['log']['enabled'] = true;

// What log driver should we use? Valid values are 'file', 'mcal',
// 'sql', and 'syslog'.
$conf['log']['type'] = 'file';

// What is the name of the log? For the 'file' driver, this is the
// path to a text file; for mcal, it would be the name of a calendar,
// and for sql it would be the table name to use. For the 'syslog'
// driver it is the facility as a _constant_ (with no quotes), e.g.:
// ... = LOG_LOCAL0;
$conf['log']['name'] = '/var/log/horde.log';

// What level of messages should we log? The values are LOG_EMERG,
// LOG_ALERT, LOG_CRIT, LOG_ERR, LOG_WARNING, LOG_NOTICE, LOG_INFO,
// and LOG_DEBUG. Each level logs itself and all those that come
// before it: LOG_ALERT would only log alerts and emergencies, but
// LOG_DEBUG would log everything.
$conf['log']['priority'] = LOG_NOTICE;

// What identifier should we use in the logs?
$conf['log']['ident'] = 'HORDE';

// Any additonal configuration information, like an MCAL or database
// username and password.
$conf['log']['params'] = array();


/**
 ** Preference System Settings
 **/

// What preferences driver should we use? Valid values are 'none'
// (meaning use system defaults and don't save any user preferences),
// 'session' (preferences only persist during the login), 'ldap', 
// and 'sql'.
$conf['prefs']['driver'] = 'sql';

// Any parameters that the preferences driver needs. This includes
// database or ldap server, username/password to connect with, etc.
$conf['prefs']['params'] = array();

// This is an example configuration for a MySQL preference backend.
// The SQL script to setup the preference database is placed in
// horde/scripts/db/prefs.sql.
$conf['prefs']['params']['phptype'] = 'mysql';
$conf['prefs']['params']['hostspec'] = 'localhost';
$conf['prefs']['params']['username'] = 'hordemgr';
$conf['prefs']['params']['password'] = 'hordemgr';
$conf['prefs']['params']['database'] = 'horde';
$conf['prefs']['params']['table'] = 'horde_prefs';

// This is an example configuration for an LDAP preference backend.
// The schemas needed for ldap are in horde/scripts/ldap.  For more
// information see sources and comments in horde/lib/Prefs/ldap.php.
//$conf['prefs']['driver'] = 'ldap';
//$conf['prefs']['params']['hostspec'] = 'localhost';
//$conf['prefs']['params']['port'] = '389';
//$conf['prefs']['params']['basedn'] = 'dc=example,dc=org';
//$conf['prefs']['params']['field'] = 'hordePrefs';
//$conf['prefs']['params']['uid'] = 'mail';
/*
 * The following is valid but would only be necessary if users
 * do NOT have permission to modify their own ldap accounts.
 */
//$conf['prefs']['params']['rootdn'] = 'cn=Manager,dc=example,dc=org';
//$conf['prefs']['params']['username'] = 'Manager';
//$conf['prefs']['params']['password'] = 'password';


/**
 ** Cache System Settings
 **/
 
// If you want to enable the Horde Cache, select a driver here.
// This is used to speed up portions of Horde by storing
// commonly processed objects to disk.
// Valid values are 'none' (don't cache any objects),
//                  'file' (store objects in filesystem)
$conf['cache']['driver'] = 'none';
// $conf['cache']['driver'] = 'file';
 
// Any parameters that the caching driver needs.
$conf['cache']['params'] = array();
// $conf['cache']['params']['dir'] = '/var/cache/horde';


/**
 ** Mailer
 **/

// What method should we use for sending mail? Valid options are
// currently 'sendmail' and 'smtp'.
$conf['mailer']['type'] = 'sendmail';

// An array holding any parameters that the Mail object will need to
// function correctly. For sendmail, this is mainly the sendmail_path
// option; SMTP requires at least a server and a port (if nonstandard).
$conf['mailer']['params'] = array();
// $conf['mailer']['params'] = array('sendmail_path' => '/usr/sbin/sendmail');
// $conf['mailer']['params'] = array('host' => 'smtp.example.com');


/**
 ** Problem Reporting
 **/

// Should we display a problem reporting link in Horde application
// menus?
$conf['problems']['enabled'] = false;

// If so, where should problem report emails be sent?
$conf['problems']['email'] = 'postmaster@example.com';


/**
 ** User Capabilities and Constraints
 **/

// Should we display help links to the user?
$conf['user']['online_help'] = true;


/**
 ** Stylesheets
 **/

// If this is true, then we will allow the browser to cache generated
// stylesheets, saving us from generating the stylesheet on every page
// request, but meaning that users will need to do a manual refresh to
// see any stylesheet changes.
$conf['css']['cached'] = true;


/**
 ** Menu settings
 **/

// If this is true, then we will provide graphical buttons in the menu
// along with text links.
$conf['menu']['images'] = true;

// Should we use DHTML to display a floating menu of Horde appliation
// links, instead of a frame?
$conf['menu']['floating_bar'] = false;

/* Which users should be treated as administrators (root, super-user)
 * by Horde? Example: $conf['auth']['admins'] = array('admin', 'john');
 * N.B.: 'admin' is used by Jonah to fetch the news.
 */
$conf['auth']['admins'] = array('admin');
